/******************************************************
*
* copyright by H.G.Kleta, 2001, 2006
*
* if you wish to use these sources, please contact
*
* fhw at indigo2 dot de
*
******************************************************/

#include "action.h"
#include "my_string.h"


/******************************************************
*
* variables
*
******************************************************/

int AgentX    = 0;
int AgentXMin = 0;
int AgentXMax = 4;

int AgentZ    = 4;
int AgentZMin = 0;
int AgentZMax = 4;

int AgentDirection = NORTH;


/******************************************************
*
* int GetAgentX(void)
*
* function returns present X position of agent
*
******************************************************/

int GetAgentX(void)
{
  return AgentX;
};/*GetAgentX()*/


/******************************************************
*
* int GetAgentZ(void)
*
* function returns present Z position of agent
*
******************************************************/

int GetAgentZ(void)
{
  return AgentZ;
};/*GetAgentZ()*/


/******************************************************
*
* int GetAgentDirection(void)
*
* function returns present Z position of agent
*
******************************************************/

int GetAgentDirection(void)
{
  return AgentDirection;
};/*GetAgentDirection()*/


/******************************************************
*
* void MoveAgent(int direction)
*
* function changes the position of the agent in the
* given direction and tests for collision with wall
*
******************************************************/

void MoveAgent(int direction)
{
  switch (direction)
    {
      case NORTH :
        AgentDirection = NORTH;
        if (AgentZ > AgentZMin)
          {
            AgentZ--;
          }/*if*/
        else
          {
            NewLine(2);
            PutString(WALL_MESSAGE);
          }/*else*/
        break;/*NORTH*/
      case EAST :
        AgentDirection = EAST;
        if (AgentX < AgentXMax)
          {
            AgentX++;
          }/*if*/
        else
          {
            NewLine(2);
            PutString(WALL_MESSAGE);
          }/*else*/
        break;/*EAST*/
      case SOUTH :
        AgentDirection = SOUTH;
        if (AgentZ < AgentZMax)
          {
            AgentZ++;
          }/*if*/
        else
          {
            NewLine(2);
            PutString(WALL_MESSAGE);
          }/*else*/
        break;/*SOUTH*/
      case WEST :
        AgentDirection = WEST;
        if (AgentX > AgentXMin)
          {
            AgentX--;
          }/*if*/
        else
          {
            NewLine(2);
            PutString(WALL_MESSAGE);
          }/*else*/
        break;/*WEST*/
/* default -> error message and exit */
      default:
        PutString(UNKNOWN_PARAMETER);
        exit(GRAMMAR_ERROR);
    }/*switch(direction)*/
};/*MoveAgent()*/


/******************************************************
*
* void TurnAgent(int value)
*
* function changes the direction of the agent for the
* given value and tests for overflow
*
******************************************************/

void TurnAgent(int value)
{
/* change direction */
  AgentDirection += value;
/* handle overflow */
  if (AgentDirection > 360)
    AgentDirection -= 360;
  if (AgentDirection < 90)
    AgentDirection += 360;
}/*TurnAgent()*/


/******************************************************
* 
* void EvaluateAnswer(char *answer1, char *answer2)
*
* function evaluates answers from prolog and calls
* functions, prints output, ... accordingly
*
******************************************************/

void EvaluateAnswer(char *answer1, char *answer2)
{
  int problem;
  int actor;
  int action;
  int parameter;

  problem  = IsInString(answer1, PROBLEM_INDICATOR);
  problem += IsInString(answer2, PROBLEM_INDICATOR);

/* set default values - used for error detection */
  actor     = 0;
  action    = 0;
  parameter = 0;

  NewLine(2);

  if (problem)
    {
      PutString(DONT_UNDERSTAND);
      PutString(PLEASE_REPHRASE);
    }/*if*/
  else
    {
/* determine actor */
      if (StringIsEqual(answer1,AGENT_STRING))
        actor = AGENT;
      if (StringIsEqual(answer1,WUMPUS_STRING))
        actor = WUMPUS;

/* determine action */
      if (IsInString(answer2,IS_STRING))
        action = IS;
      if (IsInString(answer2,GO_STRING))
        action = GO;
      if (IsInString(answer2,TURN_STRING))
        action = TURN;

/* determine parameter */
      if (IsInString(answer2,NORTH_STRING))
        parameter = NORTH;
      if (IsInString(answer2,EAST_STRING))
        parameter = EAST;
      if (IsInString(answer2,SOUTH_STRING))
        parameter = SOUTH;
      if (IsInString(answer2,WEST_STRING))
        parameter = WEST;
      if (IsInString(answer2,AHEAD_STRING))
        parameter = AHEAD;
      if (IsInString(answer2,RIGHT_STRING))
        parameter = RIGHT;
      if (IsInString(answer2,BACK_STRING))
        parameter = BACK;
      if (IsInString(answer2,LEFT_STRING))
        parameter = LEFT;
      if (IsInString(answer2,ALIVE_STRING))
        parameter = ALIVE;
      if (IsInString(answer2,NOMORE_STRING))
        parameter = NOMORE;

/* output */
      PutString("Prolog: ");
      PutString(answer1);
      NewLine(1);
      PutString("Prolog: ");
      PutString(answer2);

/****************** EVALUATE ANSWERS *****************/

      switch (actor)
        {

/*********************** AGENT ***********************/

          case AGENT :
            switch (action)
              {
/* handle death wish... */
                case IS :
                  switch (parameter)
                    {
                      case ALIVE :
                        NewLine(2);
                        PutString(AGENT_ALIVE);
                        break;/*ALIVE*/
                      case NOMORE :
                        NewLine(2);
                        PutString(AGENT_NOT_DEAD);
                        break;/*NOMORE*/
                      default:
                        NewLine(2);
                        PutString(NO_SENSE);
                    }/*switch(parameter)*/
                  break;/*IS*/
/* move agent */
                case GO :
                  switch (parameter)
                    {
                      case NORTH :
                        MoveAgent(NORTH);
                        break;/*NORTH*/
                      case EAST :
                        MoveAgent(EAST);
                        break;/*EAST*/
                      case SOUTH :
                        MoveAgent(SOUTH);
                        break;/*SOUTH*/
                      case WEST :
                        MoveAgent(WEST);
                        break;/*WEST*/
                      case AHEAD :
                        MoveAgent(AgentDirection);
                        break;/*AHEAD*/
                      case RIGHT :
                        TurnAgent(RIGHT);
                        MoveAgent(AgentDirection);
                        break;/*RIGHT*/
                      case BACK :
                        TurnAgent(BACK);
                        MoveAgent(AgentDirection);
                        TurnAgent(BACK);
                        break;/*BACK*/
                      case LEFT :
                        TurnAgent(LEFT);
                        MoveAgent(AgentDirection);
                        break;/*LEFT*/
                      default:
                        NewLine(2);
                        PutString(NO_SENSE);
                    }/*switch(parameter)*/
                  break;/*GO*/
/* turn agent */
                case TURN :
                  switch (parameter)
                    {
                      case NORTH :
                        AgentDirection = NORTH;
                        break;/*NORTH*/
                      case EAST :
                        AgentDirection = EAST;
                        break;/*EAST*/
                      case SOUTH :
                        AgentDirection = SOUTH;
                        break;/*SOUTH*/
                      case WEST :
                        AgentDirection = WEST;
                        break;/*WEST*/
                      case AHEAD :
                        NewLine(2);
                        PutString(NO_SENSE);
                        break;/*AHEAD*/
                      case RIGHT :
                        TurnAgent(RIGHT);
                        break;/*RIGHT*/
                      case BACK :
                        TurnAgent(BACK);
                        break;/*BACK*/
                      case LEFT :
                        TurnAgent(LEFT);
                        break;/*LEFT*/
                      default:
                        NewLine(2);
                        PutString(NO_SENSE);
                    }/*switch(parameter)*/
                  break;/*TURN*/
/* default -> error message and exit */
                default:
                  PutString(UNKNOWN_ACTION);
                  exit(GRAMMAR_ERROR);
              }/*switch(action)*/
            break;/*AGENT*/

/********************** WUMPUS ***********************/

          case WUMPUS :
            switch (action)
              {
                case IS :
                  switch (parameter)
                    {
                      case ALIVE :
                        NewLine(2);
                        PutString(WUMPUS_IS_ALIVE);
                        break;/*ALIVE*/
                      case NOMORE :
                        NewLine(2);
                        PutString(WUMPUS_IS_DEAD);
                        break;/*NOMORE*/
                      default:
                        NewLine(2);
                        PutString(NO_SENSE);
                    }/*switch(parameter)*/
                  break;/*IS*/
/* ignore the rest the wumpus could do... */
                default:
                  NewLine(2);
                  PutString(DONT_CARE);
              }/*switch(action)*/
            break;/*WUMPUS*/

/********************** DEFAULT **********************/

          default:
            PutString(UNKNOWN_ACTOR);
            exit(GRAMMAR_ERROR);

/*****************************************************/

        }/*switch(actor)*/
    };/*else*/

/******************* USER PROMPT *********************/

  NewLine(2);
  PutString(USER_PROMPT);

}/*EvaluateAnswer()*/


/*****************************************************/

/* END OF FILE action.c */
