/******************************************************
*
* copyright by H.G.Kleta, 2001, 2006
*
* if you wish to use these sources, please contact
*
* fhw at indigo2 dot de
*
******************************************************/

#include "my_string.h"
#include "action.h"

#include <stdio.h>
#include <string.h>

#include <GL/glut.h>


/******************************************************
*
* variables
*
******************************************************/

/* Pointer in String */
  int StringPosition;

/* user input, naked and converted */
  char UserInputString[MAX_STRING_LENGTH];
  char *UserInput = UserInputString;
  char PrologQueryString[MAX_STRING_LENGTH];
  char *PrologQuery = PrologQueryString;

/* prolog answers */
  char PrologAnswerString1[MAX_STRING_LENGTH];
  char *PrologAnswer1 = PrologAnswerString1;
  char PrologAnswerString2[MAX_STRING_LENGTH];
  char *PrologAnswer2 = PrologAnswerString2;


/******************************************************
* 
* void NewLine(int count)
* 
* function prints count times \n
* 
******************************************************/

void NewLine(int count)
{
  int i;
  for (i=0; i<count; i++)
    {
      printf("\n");
    }/*for*/
}/*NewLine()*/


/******************************************************
* 
* void PutString(char *string)
* 
* function prints string to stdout
* 
******************************************************/

void PutString(char *string)
{
  printf("%s",string);
  fflush(stdout);
}/*PutString()*/


/******************************************************
*
* void PutChar(char character)
*
* function prints character to stdout
*
******************************************************/

void PutChar(char character)
{
  printf("%c",character);
  fflush(stdout);
}/*PutChar()*/


/******************************************************
*
* int StringIsEqual(char *string1, char *string2)
*
* function returns
* 1 if string1 == string2
* 0 if string1 <> string2
*
******************************************************/

int StringIsEqual(char *string1, char *string2)
{
  if (!strcmp(string1,string2))
    {
      return 1;
    }/*if*/
  else
    {
      return 0;
    };/*else*/
}/*CompareString()*/


/******************************************************
*
* int IsInString(char *string1, char *string2)
*
* function returns 0 if string2 is not in string1,
* <> 0 else
*
******************************************************/

int IsInString(char *string1, char *string2)
{
  return (int)strstr(string1,string2);
}/*IsInString()*/


/******************************************************
* 
* void GetString(char **string)
* 
* function reads MAX_STRING_LENGTH characters from stdin
* if more characters are typed they will be discarded
* if \n is found it will be changed to \0
* 
******************************************************/

void GetString(char **string)
{
  char buffer[2];
  char *nl;

/* read input */
  fgets(*string, MAX_STRING_LENGTH, stdin);

/* if line not complete, read until newline */
  if ((nl = strchr(*string, '\n'))== NULL)
    {
      do
        {
          fgets (buffer, MAX_STRING_LENGTH , stdin);
        }/*do*/
      while (strchr (buffer, '\n') == NULL);
    }/*if*/
/* else delete newline */
  else
    {
      nl[0] = '\0';
    };/*else*/

}/*GetString()*/


/******************************************************
* 
* void ConvertString(char *InputString, char **OutputString)
* 
* function changes every blank ' ' into a comma ','  adds
* '[' at the beginning and ']' at the end, thus generating
* a list of words which is needed for the prolog-query
* 
* example: "i go north" => "[i,go,north]"
*          "THE WUMPUS IS DEAD" => "[the,wumpus,is,dead]"
*
******************************************************/

void ConvertString(char *InputString, char **OutputString)
{
  char TempString[MAX_STRING_LENGTH];
  int i;

  for (i=0; i<MAX_STRING_LENGTH; i++)
    {
      switch ( InputString[i] )
        {
          case ' ' : TempString[i] = ',';
                     break;
          default  : TempString[i] = tolower(InputString[i]);
        };/*switch*/
    }/*for*/

  *OutputString[0] = '\0';
  strcat( *OutputString , "[" );
  strcat( *OutputString , TempString );
  strcat( *OutputString , "]\0" );

}/*ConvertString()*/


/******************************************************
*
* void KeyCallback(unsigned char key, int x, int y)
*
* GL key callback function
*
* used to send key to function that evaluates input
*
******************************************************/

void KeyCallback(unsigned char key, int x, int y)
{
/* handle key */
  switch (key)
    {
/* end program */
      case ESC :
        NewLine(2);
        PutString(GOOD_BYE);
        NewLine(2);
        StopProlog();
        exit(OK);
/* evaluate user input */
      case CR :
        ConvertString(UserInput, &PrologQuery);
        CallProlog(PrologQuery, &PrologAnswer1, &PrologAnswer2);
        EvaluateAnswer(PrologAnswer1, PrologAnswer2);
        StringPosition = 0;
        UserInputString[StringPosition] = NUL;
        break;/*CR*/
/* delete last pressed key */
      case BS :
        if (StringPosition > 0)
          {
            PutChar(key);
            PutChar(SPC);
            PutChar(key);
            StringPosition--;
            UserInputString[StringPosition] = NUL;
          }/*if*/
        break;/*BS*/
/* add key to user-input-string */
      default:
        if (StringPosition < MAX_STRING_LENGTH)
          {
            PutChar(key);
            UserInputString[StringPosition] = key;
            StringPosition++;
            UserInputString[StringPosition] = NUL;
          }/*if*/
        else
          {
            NewLine(2);
            PutString(INPUT_TOO_LONG);
          }/*else*/
    }/*switch*/

}/*KeyCallback()*/


/*****************************************************/

/* END OF FILE my_string.c */
