/******************************************************
*
* copyright by H.G.Kleta, 2001, 2006
*
* if you wish to use these sources, please contact
*
* fhw at indigo2 dot de
*
******************************************************/

#ifndef __MY_STRING_H_
#define __MY_STRING_H_


/******************************************************
*
* various ASCII Codes
*
******************************************************/

#define NULL 0
#define NUL  0
#define BS   8
#define LF  10
#define CR  13
#define ESC 27
#define SPC 32


/******************************************************
*
* Exit Codes
*
******************************************************/

#define OK            0
#define PROLOG_ERROR  1
#define GRAMMAR_ERROR 2
#define GL_ERROR      3


/******************************************************
*
* various actors
*
******************************************************/

#define AGENT  1
#define WUMPUS 2

#define AGENT_STRING  "agent"
#define WUMPUS_STRING "wumpus"


/******************************************************
*
* various actions
*
******************************************************/

#define IS   1
#define GO   2
#define TURN 3

#define IS_STRING   "is"
#define GO_STRING   "go"
#define TURN_STRING "turn"


/******************************************************
*
* various parameters
*
******************************************************/

/* north, east, south and west get mathmatical values
   NOT geographical values to allow usage for rotation */
#define NORTH   360
#define EAST    270
#define SOUTH   180
#define WEST     90
/* ahead, right, back and left get negative degree values
   to allow usage for rotation and seperation from other
   parameters */
#define AHEAD  -360
#define RIGHT   -90
#define BACK   -180
#define LEFT   -270
/* handle the death of the actors */
#define ALIVE    42
#define NOMORE   66

#define NORTH_STRING  "north"
#define EAST_STRING   "east"
#define SOUTH_STRING  "south"
#define WEST_STRING   "west"
#define AHEAD_STRING  "ahead"
#define RIGHT_STRING  "right"
#define BACK_STRING   "back"
#define LEFT_STRING   "left"
#define ALIVE_STRING  "alive"
#define NOMORE_STRING "nomore"


/******************************************************
*
* various strings
*
******************************************************/

#define USER_PROMPT "Tell Prolog: "

#define GOOD_BYE "Thanks! You've made a little program very happy :-)"

#define PROBLEM_INDICATOR "_"

#define DONT_UNDERSTAND   "Sorry, I do not understand... "
#define PLEASE_REPHRASE   "Please rephrase your query..."

#define DONT_CARE         "I don't care... next query..."
#define NO_SENSE          "No-Sense-Error... next query..."

#define WALL_MESSAGE      "OUCH!"
#define AGENT_ALIVE       "Try to stay this way..."
#define AGENT_NOT_DEAD    "Wake up, your not Dead... yet :-)"
#define WUMPUS_IS_ALIVE   "Yes... It is... BE AWARE!!!"
#define WUMPUS_IS_DEAD    "Oh... poor little bugger..."

#define INPUT_TOO_LONG    "Input to long... Please press ENTER!"

#define UNKNOWN_ACTOR     "unknown actor error..."
#define UNKNOWN_ACTION    "unknown action error..."
#define UNKNOWN_PARAMETER "unknown parameter error..."


/******************************************************
*
* MAX_STRING_LENGTH
*
* maximal length of strings
*
******************************************************/

#define MAX_STRING_LENGTH 256


/******************************************************
*
* void NewLine(int count)
*
* function prints count times \n
*
******************************************************/

void NewLine(int count);


/******************************************************
*
* void PutString(char *string)
*
* function prints string to stdout
*
******************************************************/

void PutString(char *string);


/******************************************************
*
* void PutChar(char character)
*
* function prints character to stdout
*
******************************************************/

void PutChar(char character);


/******************************************************
*
* int StringIsEqual(char *string1, char *string2)
*
* function returns
* 1 if string1 == string2
* 0 if string1 <> string2
*
******************************************************/

int StringIsEqual(char *string1, char *string2);


/******************************************************
*
* int IsInString(char *string1, char *string2)
*
* function returns 0 if string2 is not in string1,
* <> 0 else
*
******************************************************/

int IsInString(char *string1, char *string2);


/******************************************************
*
* void GetString(char **string)
*
* function reads MAX_STRING_LENGTH characters from stdin
* if more characters are typed they will be discarded
* if \n is found it will be changed to \0
*
******************************************************/

void GetString(char **string);


/******************************************************
*
* void ConvertString(char *InputString, char **OutputString)
*
* function changes every blank ' ' into a comma ','  adds
* '[' at the beginning and ']' at the end, thus generating
* a list of words which is needed for the prolog-query
*
* example: "i go north" => "[i,go,north]"
*          "THE WUMPUS IS DEAD" => "[the,wumpus,is,dead]"
*
******************************************************/

void ConvertString(char *InputString, char **OutputString);


/******************************************************
*
* void KeyCallback(unsigned char key, int x, int y)
*
* GL key callback function
*
******************************************************/

void KeyCallback(unsigned char key, int x, int y);


/*****************************************************/

#else
#error "MY_STRING_H already included"
#endif /* __MY_STRING_H_ */

/* END OF FILE my_string.h */
