/******************************************************
*
* copyright by H.G.Kleta, 2001, 2006
*
* if you wish to use these sources, please contact
*
* fhw at indigo2 dot de
*
******************************************************/

#include "prolog.h"
#include "my_string.h"

#include <SWI-Prolog.h>


/******************************************************
* 
* int InitProlog(void)
* 
* function returns 0 if the initialisation of the wumpus 
* prolog environment fails, <> 0 if it succeeds
* 
******************************************************/

int InitProlog(void)
{
  char *PrologInitString[2];
  int success;

  PrologInitString[0] = "wumpus";
  PrologInitString[1] = NULL;

  success = PL_initialise(1, PrologInitString);

  return success;
}/*InitProlog()*/


/******************************************************
* 
* void CallProlog(char *query, char **answer1, char **answer2)
* 
* functions generates a prolog-query from query and returns
* two variables as strings in answer1 and answer2
* 
* example: query == "[i,go,north]"
*          prolog-query = s(Var1,Var2,[i,go,north],[]).
*          Var1 = agent
*          Var2 = go(north)
* 
******************************************************/

void CallProlog(char *query, char **answer1, char **answer2)
{
/* frame id */
  fid_t fid;
/* variables for prolog term */
  term_t PrologVar1 = PL_new_term_ref();
  term_t PrologVar2 = PL_new_term_ref();
  term_t PrologVar3 = PL_new_term_ref();
  term_t PrologVar4 = PL_new_term_ref();
  term_t PrologTerm = PL_new_term_ref();

/* open frame for prolog */
  fid = PL_open_foreign_frame();

/************ create parts of Prolog-Term ************/

/* variables to store answer */
  PL_put_variable(PrologVar1);
  PL_put_variable(PrologVar2);

/* if the string wont translate into a term, send empty list */
  if ( ! (PL_chars_to_term(query, PrologVar3)) )
    {
      PL_put_atom_chars(PrologVar3, "[]");
    }/*if*/

/* empty list to complete query - needed for DCG */
  PL_put_atom_chars(PrologVar4, "[]");

/* combine parts to complete Prolog_Term */
  PL_cons_functor(PrologTerm,
                  PL_new_functor(PL_new_atom("s"),4),
                  PrologVar1, PrologVar2, PrologVar3, PrologVar4);

/******************** Prolog Call ********************/

  PL_call(PrologTerm, NULL);

/* get answers from Prolog and store in strings */
  PL_get_chars(PrologVar1, answer1, CVT_WRITE);
  PL_get_chars(PrologVar2, answer2, CVT_WRITE);

/* close frame - discard term references */
  PL_discard_foreign_frame(fid);

}/*CallProlog()*/


/******************************************************
* 
* void StopProlog(void)
* 
* function shuts down prolog environment
* 
* IMPORTANT: use PL_cleanup() instead of PL_halt since
*            PL_halt() executes exit() and thus exits
*            main program
* 
******************************************************/

void StopProlog(void)
{
  PL_cleanup(1);
}/*StopProlog()*/


/*****************************************************/

/* END OF FILE prolog.c */
