/******************************************************
*
* copyright by H.G.Kleta, 2001, 2006
*
* if you wish to use these sources, please contact
*
* fhw at indigo2 dot de
*
******************************************************/

#include "scene.h"
#include "action.h"

#include <GL/glut.h>


/******************************************************
*
* typedefs
*
******************************************************/

typedef float CGPoint3D[3];
typedef float CGVector3D[3];


/******************************************************
*
* variables
*
******************************************************/

/* light attributes */

float lightPos[]   = {-5, 0, 5, 0};
float lightColor[] = {1, 1, 1, 1};

/* material attributes */

float matAmbient[]  = { 1.0, 0.7, 0.7};
float matDiffuse[]  = { 1.0, 0.7, 0.7};
float matSpecular[] = { 1.0, 0.7, 0.7};
float matShininess  = 20;

/* id for display list */
int CubeID;
int AgentID;

/* used points */
CGPoint3D p[] = {
                /* cube for matrix */
                  { 1, 1, 1}, { 0, 1, 1}, { 1, 1, 0}, { 0, 1, 0},
                  { 1, 0, 0}, { 0, 0, 0}, { 1, 0, 1}, { 0, 0, 1},
                /* arrow for agent */
                  { 0.5, 1,   0 },{   0, 1, 0.5 },{ 0.2, 1, 0.5 },
                  { 0.2, 1,   1 },{ 0.8, 1,   1 },{ 0.8, 1, 0.5 },
                  {   1, 1, 0.5 },
                  { 0.5, 0,   0 },{   0, 0, 0.5 },{ 0.2, 0, 0.5 },
                  { 0.2, 0,   1 },{ 0.8, 0,   1 },{ 0.8, 0, 0.5 },
                  {   1, 0, 0.5 }
                };

/* "global" variable for normal */
CGVector3D Normal;


/******************************************************
*
* void GetNormal(CGPoint3D p1, CGPoint3D p2, CGPoint3D p3)
*
* function calculates normal from points p1,p2,p3
* and stores it in "global" variable
*
******************************************************/

void GetNormal(CGPoint3D p1, CGPoint3D p2, CGPoint3D p3)
{
  Normal[0] = (p2[1]-p1[1])*(p3[2]-p2[2])-(p3[1]-p2[1])*(p2[2]-p1[2]);
  Normal[1] = (p2[2]-p1[2])*(p3[0]-p2[0])-(p3[2]-p2[2])*(p2[0]-p1[0]);
  Normal[2] = (p2[0]-p1[0])*(p3[1]-p2[1])-(p3[0]-p2[0])*(p2[1]-p1[1]);
}/*GetNormal()*/
  

/******************************************************
*
* int InitGL(void)
*
* function returns 0 if the initialisation of the GL
* environment fails, <> 0 if it succeeds
*
******************************************************/

int InitGL(void)
{
  int success;

/* initialize and display window */
  glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
  glutInitWindowSize (500, 500);
  glutInitWindowPosition (0, 0);

  success = glutCreateWindow ("My Wumpus World...");

  if (success)
    {
/* set color attributes */
      glClearColor(0.0, 0.0, 0.0, 0.0);
      glColor3f(1.0, 1.0, 1.0);

/* enable back face culling */
      glEnable(GL_CULL_FACE);
      glCullFace(GL_BACK);

/* enable automatic normalization of normal */
      glEnable(GL_NORMALIZE);

/* enable z-buffer */
      glEnable(GL_DEPTH_TEST);

/* "switch on" light */
      glEnable(GL_LIGHTING);
      glLightfv(GL_LIGHT0, GL_POSITION, lightPos);
      glLightfv(GL_LIGHT0, GL_DIFFUSE,  lightColor);
      glLightfv(GL_LIGHT0, GL_SPECULAR, lightColor);
      glEnable(GL_LIGHT0);

/* set viewattributes */
      glMatrixMode (GL_PROJECTION);
      gluPerspective(75, 1.0, 0.1, 50.0);
    }/*if*/

  return success;
};/*InitGL()*/


/******************************************************
*
* void InitObjects(void)
*
* function initialises GL objects
*
******************************************************/

void InitObjects(void)
{
/* set material attributes */
  SetMaterial();

/* generate display-list */
  CubeID  = glGenLists(1);
  AgentID = glGenLists(1);

/* cube */
  glNewList(CubeID, GL_COMPILE);

/* top */
    glBegin(GL_POLYGON);
    GetNormal(p[0],p[2],p[3]);
    glNormal3fv(Normal);
    glVertex3fv(p[0]); glVertex3fv(p[2]);
    glVertex3fv(p[3]); glVertex3fv(p[1]);
    glEnd();

/* back */
    glBegin(GL_POLYGON);
    GetNormal(p[3],p[2],p[4]);
    glNormal3fv(Normal);
    glVertex3fv(p[3]); glVertex3fv(p[2]);
    glVertex3fv(p[4]); glVertex3fv(p[5]);
    glEnd();

/* bottom */
    glBegin(GL_POLYGON);
    GetNormal(p[5],p[4],p[6]);
    glNormal3fv(Normal);
    glVertex3fv(p[5]); glVertex3fv(p[4]);
    glVertex3fv(p[6]); glVertex3fv(p[7]);
    glEnd();

/* front */
    glBegin(GL_POLYGON);
    GetNormal(p[7],p[6],p[0]);
    glNormal3fv(Normal);
    glVertex3fv(p[7]); glVertex3fv(p[6]);
    glVertex3fv(p[0]); glVertex3fv(p[1]);
    glEnd();

/* right */
    glBegin(GL_POLYGON);
    GetNormal(p[0],p[6],p[4]);
    glNormal3fv(Normal);
    glVertex3fv(p[0]); glVertex3fv(p[6]);
    glVertex3fv(p[4]); glVertex3fv(p[2]);
    glEnd();

/* left */
    glBegin(GL_POLYGON);
    GetNormal(p[1],p[3],p[5]);
    glNormal3fv(Normal);
    glVertex3fv(p[1]); glVertex3fv(p[3]);
    glVertex3fv(p[5]); glVertex3fv(p[7]);
    glEnd();

  glEndList();

/* Agent */
  glNewList(AgentID, GL_COMPILE);

/* top */
    glBegin(GL_POLYGON);
    GetNormal(p[8],p[9],p[10]);
    glNormal3fv(Normal);
    glVertex3fv(p[8]);  glVertex3fv(p[9]);
    glVertex3fv(p[10]); glVertex3fv(p[11]);
    glVertex3fv(p[12]); glVertex3fv(p[13]);
    glVertex3fv(p[14]);
    glEnd();

/* bottom */
    glBegin(GL_POLYGON);
    GetNormal(p[15],p[21],p[20]);
    glNormal3fv(Normal);
    glVertex3fv(p[15]); glVertex3fv(p[21]);
    glVertex3fv(p[20]); glVertex3fv(p[19]);
    glVertex3fv(p[18]); glVertex3fv(p[17]);
    glVertex3fv(p[16]);
    glEnd();

    glBegin(GL_POLYGON);
    GetNormal(p[11],p[18],p[19]);
    glNormal3fv(Normal);
    glVertex3fv(p[11]); glVertex3fv(p[18]);
    glVertex3fv(p[19]); glVertex3fv(p[12]);
    glEnd();

    glBegin(GL_POLYGON);
    GetNormal(p[10],p[17],p[18]);
    glNormal3fv(Normal);
    glVertex3fv(p[10]); glVertex3fv(p[17]);
    glVertex3fv(p[18]); glVertex3fv(p[11]);
    glEnd();

    glBegin(GL_POLYGON);
    GetNormal(p[9],p[16],p[17]);
    glNormal3fv(Normal);
    glVertex3fv(p[9]);  glVertex3fv(p[16]);
    glVertex3fv(p[17]); glVertex3fv(p[10]);
    glEnd();

    glBegin(GL_POLYGON);
    GetNormal(p[8],p[15],p[16]);
    glNormal3fv(Normal);
    glVertex3fv(p[8]);  glVertex3fv(p[15]);
    glVertex3fv(p[16]); glVertex3fv(p[9]);
    glEnd();

    glBegin(GL_POLYGON);
    GetNormal(p[14],p[21],p[15]);
    glNormal3fv(Normal);
    glVertex3fv(p[14]); glVertex3fv(p[21]);
    glVertex3fv(p[15]); glVertex3fv(p[8]);
    glEnd();

    glBegin(GL_POLYGON);
    GetNormal(p[13],p[20],p[21]);
    glNormal3fv(Normal);
    glVertex3fv(p[13]); glVertex3fv(p[20]);
    glVertex3fv(p[21]); glVertex3fv(p[14]);
    glEnd();

    glBegin(GL_POLYGON);
    GetNormal(p[12],p[19],p[20]);
    glNormal3fv(Normal);
    glVertex3fv(p[12]); glVertex3fv(p[19]);
    glVertex3fv(p[20]); glVertex3fv(p[13]);
    glEnd();

  glEndList();

}/*InitObjects()*/


/******************************************************
*
* void SetMaterial(void)
*
* function sets material attributes
*
******************************************************/

void SetMaterial(void)
{
/* set material attributes */
  glMaterialfv(GL_FRONT, GL_AMBIENT,    matAmbient);
  glMaterialfv(GL_FRONT, GL_DIFFUSE,    matDiffuse);
  glMaterialfv(GL_FRONT, GL_SPECULAR,   matSpecular);
  glMaterialfv(GL_FRONT, GL_SHININESS, &matShininess);
};/*SetMaterial()*/


/******************************************************
*
* void DrawWorld(void)
*
* function draws the graphical wumpus world
*
******************************************************/

void DrawWorld(void)
{
/* position of agent */
  int AgentX;
  int AgentZ;
  int AgentDirection;

/* initialize buffer */
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

/* init modeling matrix-stack */
  glMatrixMode(GL_MODELVIEW);

/* origin */
  glLoadIdentity();

/* set look-at */
  gluLookAt( 2.5, 0, 10,
             2.5, 0, 0,
             0, 1, 0);

/* rotation for better view */
  glRotatef(30,1,0,0);

  glPushMatrix();
  glPushMatrix();
  glPushMatrix();

/* matrix */
  glPopMatrix();
  glTranslatef( 0, 0.1, 4.975);
  glScalef( 5.0, 0.05, 0.05);
  glCallList(CubeID);

  glTranslatef( 0, 0, -20);
  glCallList(CubeID);
  glTranslatef( 0, 0, -20);
  glCallList(CubeID);
  glTranslatef( 0, 0, -20);
  glCallList(CubeID);
  glTranslatef( 0, 0, -20);
  glCallList(CubeID);
  glTranslatef( 0, 0, -20);
  glCallList(CubeID);

  glPopMatrix();
  glRotatef(90,0,1,0);
  glTranslatef( -5, 0.1, 4.975);
  glScalef( 5.0, 0.05, 0.05);
  glCallList(CubeID);

  glTranslatef( 0, 0, -20);
  glCallList(CubeID);
  glTranslatef( 0, 0, -20);
  glCallList(CubeID);
  glTranslatef( 0, 0, -20);
  glCallList(CubeID);
  glTranslatef( 0, 0, -20);
  glCallList(CubeID);
  glTranslatef( 0, 0, -20);
  glCallList(CubeID);

/* agent */
  glPopMatrix();

/* position of agent */
  AgentX = GetAgentX();
  AgentZ = GetAgentZ();
  glTranslatef((float)AgentX,0,(float)AgentZ);

/* rotation of agent (at agents center) */
  glTranslatef(0.5,0.0,0.5);
  AgentDirection = GetAgentDirection();
  glRotatef(AgentDirection,0,1,0);
  glTranslatef(-0.5,0.0,-0.5);

/* center agent position relative to matrix */
  glTranslatef(0.2,0.0,0.2);

  glScalef( 0.6, 0.6, 0.6);
  glCallList(AgentID);
   
/* display objects */
  glFlush();

/* swap front- / back-buffer */
  glutSwapBuffers();
}/*DrawWorld()*/


/*****************************************************/

/* END OF FILE scene.c */
